@@ -24,6 +24,9 @@
 #include "dialog_layout.h"
 #include "treestore_layout.h"
 
+namespace Glom
+{
+
 class Dialog_Layout_Details : public Dialog_Layout
 {
 public:
@@ -49,10 +52,10 @@ protected:
   virtual void save_to_document();
 
   sharedptr<Relationship> offer_relationship_list();
+  sharedptr<Relationship> offer_relationship_list(const sharedptr<const Relationship>& relationship);
 
   Gtk::TreeModel::iterator get_selected_group_parent() const;
   sharedptr<LayoutItem_Button> offer_button_script_edit(const sharedptr<const LayoutItem_Button>& button);
-  //sharedptr<LayoutItem_Text> offer_textobject_edit(const sharedptr<const LayoutItem_Text>& textobject);
 
   //signal handlers:
   virtual void on_button_field_up();
@@ -64,6 +67,7 @@ protected:
   virtual void on_button_add_related();
   virtual void on_button_add_button();
   virtual void on_button_add_text();
+  virtual void on_button_add_image();
   virtual void on_button_field_formatting();
   virtual void on_button_edit();
   virtual void on_treeview_fields_selection_changed();
@@ -89,6 +93,7 @@ protected:
   Gtk::Button* m_button_add_related;
   Gtk::Button* m_button_add_button;
   Gtk::Button* m_button_add_text;
+  Gtk::Button* m_button_add_image;
   Gtk::Button* m_button_field_delete;
   Gtk::Button* m_button_field_formatting;
   Gtk::Button* m_button_edit;
@@ -97,4 +102,6 @@ protected:
   Glib::RefPtr<TreeStore_Layout> m_model_items;
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_DIALOG_LAYOUT_DETAILS_H
