@@ -29,12 +29,6 @@
 namespace Glom
 {
 
-//JPEG seems to give ugly results when saved to the database and shown again.
-//#define GLOM_IMAGE_FORMAT "jpeg"
-//#define GLOM_IMAGE_FORMAT_MIME_TYPE "image/jpeg"
-#define GLOM_IMAGE_FORMAT "png"
-#define GLOM_IMAGE_FORMAT_MIME_TYPE "image/png"
-
 ImageGlom::ImageGlom()
 : m_image(Gtk::Stock::MISSING_IMAGE, Gtk::ICON_SIZE_DIALOG), //The widget is invisible if we don't specify an image.
   m_pMenuPopup_UserMode(0)
@@ -147,89 +141,13 @@ void ImageGlom::set_pixbuf(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
 
 void ImageGlom::set_value(const Gnome::Gda::Value& value)
 {
-  bool pixbuf_set = false;
-
-  if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Conversions::get_pixbuf_for_gda_value(value);
+  if(pixbuf)
   {
-    glong size = 0;
-    const gpointer pData = value.get_binary(size);
-    if(size && pData)
-    {
-      //libgda does not currently properly unescape binary data,
-      //so pData is actually a null terminated string, of escaped binary data.
-      //This workaround should be removed when libgda is fixed:
-      //(It is fixed in libgd-2.0 but is unlikely to be fixed in libgda-1.2)
-      size_t buffer_binary_length = 0;
-      guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)pData /* must be null-terminated */, &buffer_binary_length); //freed by us later.
-      if(buffer_binary)
-      {
-        //typedef std::list<Gdk::PixbufFormat> type_list_formats;
-        //const type_list_formats formats = Gdk::Pixbuf::get_formats();
-        //std::cout << "Debug: Supported pixbuf formats:" << std::endl;
-        //for(type_list_formats::const_iterator iter = formats.begin(); iter != formats.end(); ++iter)
-        //{
-        //  std::cout << " name=" << iter->get_name() << ", writable=" << iter->is_writable() << std::endl;
-        //}
-
-        Glib::RefPtr<Gdk::PixbufLoader> refPixbufLoader;
-
-        // PixbufLoader::create() is broken in gtkmm before 2.6.something,
-        // so let's do this in C so it works with all 2.6 versions:
-        GError* error = 0;
-        GdkPixbufLoader* loader = gdk_pixbuf_loader_new_with_type(GLOM_IMAGE_FORMAT, &error);
-        if(!error)
-          refPixbufLoader = Glib::wrap(loader);
-        else
-          std::cerr << "ImageGlom::set_value(): Error while calling gdk_pixbuf_loader_new_with_type()." << std::endl;
-
-        /*
-        try
-        {
-          refPixbufLoader = Gdk::PixbufLoader::create(GLOM_IMAGE_FORMAT);
-          g_warning("debug a1");
-        }
-        catch(const Gdk::PixbufError& ex)
-        {
-          refPixbufLoader.clear();
-          g_warning("PixbufLoader::create failed: %s",ex.what().c_str());
-        }
-        */
-
-        if(refPixbufLoader)
-        {
-          try
-          {
-            guint8* puiData = (guint8*)buffer_binary;
-
-            //g_warning("ImageGlom::set_value(): debug: from db: ");
-            //for(int i = 0; i < 10; ++i)
-            //  g_warning("%02X (%c), ", (guint8)puiData[i], (char)puiData[i]);
-
-            refPixbufLoader->write(puiData, (glong)buffer_binary_length);
-
-            set_pixbuf(refPixbufLoader->get_pixbuf());
-            pixbuf_set = true;
-
-            scale();
-
-            refPixbufLoader->close(); //This throws if write() threw, so it must be inside the try block.
-          }
-          catch(const Glib::Exception& ex)
-          {
-            g_warning("ImageGlom::set_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
-          }
-
-          free(buffer_binary);
-        }
-      }
-
-      //TODO: load the image, using the mime type stored elsewhere.
-      //pixbuf = Gdk::Pixbuf::create_from_data(
-    }
-
+    set_pixbuf(pixbuf);
+    scale();
   }
-
-  if(!pixbuf_set)
+  else
   {
     /*
     std::cout << "Debug: Setting MISSING_IMAGE" << std::endl;
@@ -239,7 +157,7 @@ void ImageGlom::set_value(const Gnome::Gda::Value& value)
     Glib::RefPtr<Gtk::Style> style = get_style();
     if(style)
     {
-      std::cout << "Debug: Setting MISSING_IMAGE 3" << std::endl;
+      /std::cout << "Debug: Setting MISSING_IMAGE 3" << std::endl;
 
       const Gtk::IconSet iconset = style->lookup_icon_set(Gtk::Stock::MISSING_IMAGE);
 
